package com.sztzjy.fund_investment.controller;

import com.github.pagehelper.PageInfo;
import com.sztzjy.fund_investment.annotation.AnonymousAccess;
import com.sztzjy.fund_investment.entity.FoundProject;
import com.sztzjy.fund_investment.entity.ProjectPool;
import com.sztzjy.fund_investment.mapper.FoundProjectMapper;
import com.sztzjy.fund_investment.service.ISysFoundProjectService;
import com.sztzjy.fund_investment.service.ISysProjectPoolService;
import com.sztzjy.fund_investment.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.nio.file.Paths;

//yz
@RestController
@RequestMapping("/stu/foundProjectController")
@Api(tags = "学生端--寻找项目")
public class FoundProjectController {
    @Autowired
    ISysProjectPoolService projectPoolService;
    @Value("${file.path}")
    private String filePath;
    @Autowired
    FoundProjectMapper foundProjectMapper;
    @Autowired
    ISysFoundProjectService foundProjectService;

    @GetMapping("getProjectPoolList")
    @ApiOperation("寻找项目数据展示")
    @AnonymousAccess
    public ResultEntity getFundraising(@ApiParam("条数") @RequestParam Integer size,
                                       @ApiParam("页数") @RequestParam Integer index) {
        PageInfo<ProjectPool> pageInfo=projectPoolService.selectList(size,index);
        return new ResultEntity(HttpStatus.OK,"寻找项目数据展示成功",pageInfo);
    }


    @GetMapping("getBP")
    @ApiOperation("BP按钮")
    public ResponseEntity<FileSystemResource> getFundraising(@ApiParam("bp路径") @RequestParam String bp_path) {
        String PDFPath = filePath + bp_path;
        File videoFile = new File(PDFPath);
        if (videoFile.exists()) {
            Paths.get(PDFPath);
            FileSystemResource fileSystemResource = new FileSystemResource(videoFile);
            return ResponseEntity.ok()
                    .contentType(MediaType.parseMediaType("application/pdf"))
                    .body(fileSystemResource);
        } else {
            return ResponseEntity.notFound().build();
        }
    }

    @PostMapping("projectApproval")
    @ApiOperation("立项")
    @AnonymousAccess
    public ResultEntity projectApproval(@ApiParam("传递flow_id,project_pool_id,found_project_conclusion") @RequestBody FoundProject foundProject){
        if(StringUtils.isBlank(foundProject.getFlowId())){
            return new ResultEntity(HttpStatus.BAD_REQUEST,"流程ID未上传");
        }
        if(StringUtils.isBlank(foundProject.getProjectPoolId())){
            return new ResultEntity(HttpStatus.BAD_REQUEST,"未选择项目");
        }
        if(StringUtils.isBlank(foundProject.getFoundProjectConclusion())){
            return new ResultEntity(HttpStatus.BAD_REQUEST,"未填写选择理由");
        }
        FoundProject foundProject1 = foundProjectMapper.selectByPrimaryKey(foundProject.getFlowId());
        if(foundProject1!=null){
            return new ResultEntity(HttpStatus.BAD_REQUEST,"本次实训已立项过");
        }
        foundProjectMapper.insert(foundProject);
        return new ResultEntity(HttpStatus.OK,"立项成功");
    }


    @GetMapping("getProjectApproval")
    @ApiOperation("立项回显")
    public ResultEntity getProjectApproval(@ApiParam("流程ID") @RequestParam String flowId){
        FoundProject foundProject=foundProjectService.selectByFlowId(flowId);
        foundProject.setFinancialDueDiligence(null);
        foundProject.setServiceDueDiligence(null);
        return new ResultEntity(HttpStatus.OK,"立项回显成功",foundProject);
    }


}
