package com.sztzjy.fund_investment.controller;

import com.sztzjy.fund_investment.entity.Fundraising;
import com.sztzjy.fund_investment.mapper.FundraisingMapper;
import com.sztzjy.fund_investment.service.ISysFundraisingService;
import com.sztzjy.fund_investment.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.UUID;

//yz
@RestController
@RequestMapping("/stu/fundraisingController")
@Api(tags = "学生端--资金募资")
public class FundraisingController {
    @Autowired
    ISysFundraisingService fundraisingService;
    @Autowired
    FundraisingMapper fundraisingMapper;

    @GetMapping("getFundraising")
    @ApiOperation("资金募资回显")
    public ResultEntity getFundraising(@ApiParam("流程id") @RequestParam String flowId) {
        Fundraising fundraising=fundraisingService.selectByFlowId(flowId);
        return new ResultEntity(HttpStatus.OK,"资金募资回显成功",fundraising);
    }

    @GetMapping("insertFundraising")
    @ApiOperation("新增基金募资")
    public ResultEntity insertFundraising(@ApiParam("流程id") @RequestParam String flowId,
                                          @ApiParam("自有资金") @RequestParam BigDecimal ownFunds,
                                          @ApiParam("银行借款") @RequestParam BigDecimal bankLoan,
                                          @ApiParam("优先级LP") @RequestParam(required = false) BigDecimal priorityLP,
                                          @ApiParam("劣后级LP") @RequestParam(required = false) BigDecimal subordinatedLP,
                                          @ApiParam("基金募资金额（为前面四个参数之和）") @RequestParam BigDecimal fundraisingAmount) {
        if (ownFunds.add(bankLoan).compareTo(BigDecimal.valueOf(1000000))<0){
            return new ResultEntity(HttpStatus.BAD_REQUEST,"新增基金募资失败-自有资金与银行借款之和不小于一百万");
        }
        if(fundraisingService.selectByFlowId(flowId)!=null){
            return new ResultEntity(HttpStatus.BAD_REQUEST,"新增基金募资失败-基金募资已提交");
        }
        Fundraising fundraising = new Fundraising();
        fundraising.setId(String.valueOf(UUID.randomUUID()));
        fundraising.setFlowId(flowId);
        fundraising.setOwnFunds(ownFunds);
        fundraising.setBankLoan(bankLoan);
        fundraising.setPriorityLp(priorityLP);
        fundraising.setSubordinatedLp(subordinatedLP);
        fundraising.setFundraisingAmount(fundraisingAmount);
        Boolean aBoolean = fundraisingService.insertFundraising(fundraising);
        if(aBoolean){
            return new ResultEntity(HttpStatus.OK,"资金募资新增成功",fundraising);
        }else {
            return new ResultEntity(HttpStatus.BAD_REQUEST,"新增基金募资失败");
        }

    }
}
