package com.sztzjy.fund_investment.controller;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.nimbusds.jose.shaded.gson.JsonObject;
import com.sztzjy.fund_investment.annotation.AnonymousAccess;
import com.sztzjy.fund_investment.entity.AllotmentObject;
import com.sztzjy.fund_investment.entity.AllotmentObjectExample;
import com.sztzjy.fund_investment.entity.dto.AllotmentObjectDTO;
import com.sztzjy.fund_investment.mapper.AllotmentObjectMapper;
import com.sztzjy.fund_investment.service.InquiryIssuanceService;
import com.sztzjy.fund_investment.util.ResultEntity;
import com.sztzjy.fund_investment.util.excel.FilePortUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.checkerframework.checker.units.qual.A;
import org.springframework.data.domain.PageImpl;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author tz
 * @date 2023/12/5 8:46
 */
@RestController
@RequestMapping("/inquiryIssuance")
@Api(tags = "学生端--询价发行")
public class InquiryIssuanceController {
    @Resource
    InquiryIssuanceService inquiryIssuanceService;
    @Resource
    AllotmentObjectMapper allotmentObjectMapper;
    @AnonymousAccess
    @GetMapping("/queryInquiryInvestors")
    @ApiOperation("询价投资者查看")
    public ResultEntity<PageInfo<AllotmentObjectDTO>> queryInquiryInvestors(@ApiParam("流程ID") String flowId,
                                                                            @ApiParam("每页条数") Integer size,
                                                                            @ApiParam("当前页") Integer index){
        PageInfo<AllotmentObjectDTO> pageInfo=inquiryIssuanceService.queryInquiryInvestors(flowId,size,index);
        return new ResultEntity<>(HttpStatus.OK,"成功",pageInfo);
    }

//    @AnonymousAccess
//    @GetMapping("/queryBuiltIn")
//    @ApiOperation("内置数据查看")
//    public ResultEntity<List<AllotmentObject>> queryBuiltIn(String flowId){
//        List<AllotmentObject> list=inquiryIssuanceService.queryBuiltIn(flowId);
//        return new ResultEntity<>(HttpStatus.OK,"成功",list);
//    }

    @AnonymousAccess
    @GetMapping("/insertInquiryInvestors")
    @ApiOperation("询价咨询者增加所选")
    public ResultEntity insertInquiryInvestors(@ApiParam("所选ID") String[] ids,
                                               @ApiParam("流程ID") String flowId){
        inquiryIssuanceService.insertInquiryInvestors(ids,flowId);
        return new ResultEntity<>(HttpStatus.OK,"新增成功");
    }
    @AnonymousAccess
    @GetMapping("/deleteInquiryInvestors")
    @ApiOperation("询价咨询者删除所选")
    public ResultEntity deleteInquiryInvestors(@ApiParam("所选ID") String[] ids,
                                               @ApiParam("流程ID") String flowId){
        inquiryIssuanceService.deleteInquiryInvestors(ids,flowId);
        return new ResultEntity<>(HttpStatus.OK,"删除成功");
    }

    @AnonymousAccess
    @GetMapping("/identifiedInvestorsExport")
    @ApiOperation("导出已确定投资者")
    public void filePort(HttpServletResponse response, @RequestParam String flowId) {
        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"配售对象编码", "配售对象名称", "配售对象类型", "投资者编码","投资者名称","投资者类型"};

        //实际数据结果集
        AllotmentObjectExample allotmentObjectExample = new AllotmentObjectExample();
        allotmentObjectExample.createCriteria().andFlowIdEqualTo(flowId).andLogicEqualTo(1);
        List<AllotmentObject> allotmentObjectList = allotmentObjectMapper.selectByExample(allotmentObjectExample);

        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("allotmentObjectCode", "allotmentObjectName", "allotmentObjectType", "investorCode","investorName","investorType");
        try {
            FilePortUtil.exportExcel(response, title, headers, allotmentObjectList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
