package com.sztzjy.fund_investment.controller;

import com.sztzjy.fund_investment.annotation.AnonymousAccess;
import com.sztzjy.fund_investment.entity.ExitTime;
import com.sztzjy.fund_investment.entity.ProfitManagement;
import com.sztzjy.fund_investment.entity.ProfitManagementExample;
import com.sztzjy.fund_investment.entity.TrainingReport;
import com.sztzjy.fund_investment.mapper.ExitTimeMapper;
import com.sztzjy.fund_investment.mapper.ProfitManagementMapper;
import com.sztzjy.fund_investment.service.InvestmentReportService;
import com.sztzjy.fund_investment.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;


@RestController
@RequestMapping("/stu/report")
@Api(tags = "学生端--投资报告")
public class InvestmentReportController {
    @Autowired
    private InvestmentReportService investmentReportService;
    @Autowired
    private ExitTimeMapper exitTimeMapper;
    @Autowired
    ProfitManagementMapper profitManagementMapper;

    @AnonymousAccess
    @GetMapping("getProjectPoolList")
    @ApiOperation("股权投资收益率")
    public ResultEntity<Map<Integer, BigDecimal>> getFundraising(@RequestParam String flowId) {
        ProfitManagementExample profitManagementExample = new ProfitManagementExample();
        profitManagementExample.createCriteria().andFlowIdEqualTo(flowId);
        profitManagementExample.setOrderByClause("market_time ASC");
        List<ProfitManagement> profitManagementList = profitManagementMapper.selectByExample(profitManagementExample);
        Map<Integer, BigDecimal> map = new LinkedHashMap<>();
        for (ProfitManagement profitManagement : profitManagementList) {
            if (profitManagement.getFundEarnings() != null && profitManagement.getFundEarnings().compareTo(BigDecimal.ZERO) != 0
                    && profitManagement.getInvestmentAmount() != null && profitManagement.getInvestmentAmount().compareTo(BigDecimal.ZERO) != 0) {
                BigDecimal fundEarnings = profitManagement.getFundEarnings(); // 基金收益
                BigDecimal investmentAmount = profitManagement.getInvestmentAmount(); // 投资金额

                // 基金收益/投资金额=投资收益率
                BigDecimal bigDecimal = fundEarnings.divide(investmentAmount,6,RoundingMode.HALF_UP);
                map.put(profitManagement.getMarketTime(), bigDecimal);
            }
        }
        return new ResultEntity<Map<Integer, BigDecimal>>(map);
    }


    @AnonymousAccess
    @GetMapping("getExitData")
    @ApiOperation("左侧退出时数据")
    public ResultEntity<ExitTime> getExitData(@RequestParam String flowId) {
        return new ResultEntity<ExitTime>(exitTimeMapper.selectByPrimaryKey(flowId));
    }


    @AnonymousAccess
    @PostMapping("commitExperience")
    @ApiOperation("提交心得")
    public ResultEntity<String> commitExperience(@ApiParam("传三个ID、experience、uploadTime") @RequestBody TrainingReport trainingReport) {
        return investmentReportService.commitExperience(trainingReport);
    }

    @AnonymousAccess
    @PostMapping("getExperience")
    @ApiOperation("心得回显")
    public ResultEntity<String> getExperience(@RequestBody String flowId) {
        List<TrainingReport> trainingReports = investmentReportService.getTrainingReports(flowId);
        if (trainingReports.isEmpty()) {
            return null;
        }
        TrainingReport trainingReport = trainingReports.get(0);
        if (StringUtils.isBlank(trainingReport.getExperience())) {
            return null;
        }
        return new ResultEntity<>(trainingReport.getExperience());
    }

    @AnonymousAccess
    @PostMapping("uploadReport")
    @ApiOperation("上传报告")
    public ResultEntity<String> uploadReport(@RequestParam("file") @RequestPart(name = "file") MultipartFile file,
                                             @ApiParam("文件名称") @RequestParam String fileName,
                                             @RequestParam String flowId,
                                             @RequestParam String schoolId) {
        return investmentReportService.uploadReport(file, fileName, flowId, schoolId);
    }

}
