package com.sztzjy.fund_investment.controller;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.fund_investment.annotation.AnonymousAccess;
import com.sztzjy.fund_investment.config.Constant;
import com.sztzjy.fund_investment.entity.*;
import com.sztzjy.fund_investment.mapper.FlowMapper;
import com.sztzjy.fund_investment.mapper.PerformanceScoreMapper;
import com.sztzjy.fund_investment.mapper.TopicsMapper;
import com.sztzjy.fund_investment.mapper.UserTableMapper;
import com.sztzjy.fund_investment.service.TopicService;
import com.sztzjy.fund_investment.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


/**
 * @Author xcj
 * @Date 2023/11/14
 */
@RestController
@RequestMapping("/stu/topics")
@Api(tags = "学生端--答题视频相关接口")
public class TopicController {
    @Autowired
    private TopicsMapper topicsMapper;
    @Autowired
    private TopicService topicService;
    @Autowired
    private PerformanceScoreMapper performanceScoreMapper;
    @Autowired
    private UserTableMapper userTableMapper;
    @Autowired
    private FlowMapper flowMapper;

    /* 导入功能
     * @author xcj
     * @Date 2023/11/22
     */
    @PostMapping("importFile")
    @ApiOperation("批量导入题目功能")
    @AnonymousAccess
    public ResultEntity<Integer> getRandomTopic(@ApiParam("导入的文件") @RequestParam MultipartFile file,
                                                @ApiParam("学校ID") @RequestParam String schoolId,
                                                @ApiParam("知识测评题库,项目尽调题库,项目估值题库,IPO上市板块题目,IPO上市条件题目") @RequestParam String module) {
        return new ResultEntity<Integer>(topicService.importFile(file, schoolId, module));
    }


    /*
     * @author xcj
     * @Date 2023/11/14
     */
    @GetMapping("getRandomTopic")
    @ApiOperation("知识测评,项目估值,项目尽调,答题页面返回随机6条数据")
    @AnonymousAccess
    public ResultEntity<List<TopicsWithBLOBs>> getRandomTopic(@ApiParam("题目来源模块:知识测评题库,项目尽调题库,项目估值题库") @RequestParam String module) {
        return new ResultEntity<List<TopicsWithBLOBs>>(topicService.getRandomTopic(module));
    }


    /*
     * @author xcj
     * @Date 2023/11/21
     */
    @GetMapping("getIPOTopicTwo")
    @ApiOperation("IPO页面弹窗2使用返回随机6条数据")
    @AnonymousAccess
    public ResultEntity<List<TopicsWithBLOBs>> getIPOTopicTwo() {
        return new ResultEntity<List<TopicsWithBLOBs>>(topicService.getIPOTopicTwo());
    }


    /*
     * @author xcj
     * @Date 2023/11/15
     */
    @PostMapping("getRightScore/{flowId}")
    @ApiOperation("知识测评,项目估值,项目尽调,IPO申请,判断答案对错算分")
    @AnonymousAccess
    public ResultEntity<List<String>> getRightScore(@ApiParam("需要ID和用户填写的答案") @PathVariable("flowId") String flowId,
                                                    @RequestBody List<TopicsWithBLOBs> topics) {
        return topicService.getRightScore(topics, flowId);
    }


    /*
     * @author xcj
     * @Date 2023/11/15
     */
    @PostMapping("getTopicRecord")
    @ApiOperation("知识测评、项目估值、项目尽调,IPO申请,回显答题记录")
    @AnonymousAccess
    public ResultEntity<List<TopicRecord>> getTopicRecord(@ApiParam("getRightScore返回的ID集合") @RequestBody List<String> ids) {
        return new ResultEntity<List<TopicRecord>>(topicService.getTopicRecord(ids));
    }

    /*
     * @author xcj
     * @Date 2023/11/21
     */
    @GetMapping("getAllScore")
    @ApiOperation("页面回显得分通用接口")
    @AnonymousAccess
    public ResultEntity<PerformanceScore> getAllScore(@RequestParam String flowId) {
        PerformanceScoreExample performanceScoreExample = new PerformanceScoreExample();
        performanceScoreExample.createCriteria().andFlowIdEqualTo(flowId);
        List<PerformanceScore> performanceScores = performanceScoreMapper.selectByExample(performanceScoreExample);
        return new ResultEntity<>(performanceScores.get(0));
    }


    @GetMapping("getIPOTopicOne")
    @ApiOperation("IPO弹窗1")
    @AnonymousAccess
    public ResultEntity<TopicsWithBLOBs> getIPOTopicOne() {
        TopicsWithBLOBs topics = topicsMapper.selectByPrimaryKey(Constant.NINE);
        return new ResultEntity<TopicsWithBLOBs>(topics);
    }


    @AnonymousAccess
    @GetMapping("/getVideoScore")
    @ApiOperation("实务认知计算视频得分接口")
    public void getVideoScore(@ApiParam("观看视频时间，单位秒") @RequestParam Integer time,
                              @RequestParam String flowId,
                              @RequestParam String schoolId) {

        //系统的第一个流程，新增一个得分对象
        PerformanceScore performanceScore = new PerformanceScore();

        //观看2分钟以上得满分5分，否则不得分
        BigDecimal score;
        if (time >= 120) {
            score = BigDecimal.valueOf(5);
            performanceScore.setPracticalCognitionScore(score);
        } else {
            performanceScore.setPracticalCognitionScore(BigDecimal.ZERO);
        }
        performanceScore.setId(IdUtil.simpleUUID());
        performanceScore.setSchoolId(schoolId);
        performanceScore.setFlowId(flowId);
        FlowExample flowExample = new FlowExample();
        flowExample.createCriteria().andFlowIdEqualTo(flowId);
        List<Flow> flows = flowMapper.selectByExample(flowExample);
        Flow flow = flows.get(0);
        UserTable userTable = userTableMapper.selectByPrimaryKey(flow.getUserid());
        performanceScore.setClassId(userTable.getClassId());
        performanceScore.setPracticalCognitionTime(new Date());

        //防止后面算分有问题，在第一个步骤初始化成0
        performanceScore.setKnowledgeAssessmentScore(BigDecimal.ZERO);
        performanceScore.setFundraisingScore(BigDecimal.ZERO);
        performanceScore.setProjectSearchScore(BigDecimal.ZERO);
        performanceScore.setProjectSearchReportScore(BigDecimal.ZERO);
        performanceScore.setProjectDueDiligenceScore(BigDecimal.ZERO);
        performanceScore.setProjectValuationEstimationRiskScore(BigDecimal.ZERO);
        performanceScore.setProjectDueDiligenceBusinessReportScore(BigDecimal.ZERO);
        performanceScore.setProjectDueDiligenceFinanceReportScore(BigDecimal.ZERO);
        performanceScore.setProjectValuationRelativeScore(BigDecimal.ZERO);
        performanceScore.setProjectValuationScore(BigDecimal.ZERO);
        performanceScore.setProjectValuationAbsoluteScore(BigDecimal.ZERO);
        performanceScore.setProjectUseValuationAbsoluteScore(BigDecimal.ZERO);
        performanceScore.setProjectUseValuationRelativeScore(BigDecimal.ZERO);
        performanceScore.setInvestmentSigningVlauationReportScore(BigDecimal.ZERO);
        performanceScore.setInvestmentSigningDiligenceReportScore(BigDecimal.ZERO);
        performanceScore.setInvestmentSigningInvestScore(BigDecimal.ZERO);
        performanceScore.setInvestmentSigningScore(BigDecimal.ZERO);
        performanceScore.setIpoFlowScore(BigDecimal.ZERO);
        performanceScore.setIpoListedConditionScore(BigDecimal.ZERO);
        performanceScore.setIpoListedSectorScore(BigDecimal.ZERO);
        performanceScore.setNewPricingIssuanceScore(BigDecimal.ZERO);
        performanceScore.setPricingIssuanceConfirmScore(BigDecimal.ZERO);
        performanceScore.setInitPricingIssuanceScore(BigDecimal.ZERO);
        performanceScore.setPricingIssuanceEnteringScore(BigDecimal.ZERO);
        performanceScore.setProfitManagementMarketValueScore(BigDecimal.ZERO);
        performanceScore.setProfitManagementFundScore(BigDecimal.ZERO);
        performanceScore.setProfitManagementFundExitScore(BigDecimal.ZERO);
        performanceScore.setProfitManagementMarketValueExitScore(BigDecimal.ZERO);
        performanceScore.setProfitDistributionScore(BigDecimal.ZERO);
        performanceScore.setInvestmentReportScore(BigDecimal.ZERO);
        performanceScore.setExperienceScore(BigDecimal.ZERO);
        performanceScore.setTotalScore(BigDecimal.ZERO);
        performanceScoreMapper.insert(performanceScore);
    }


}
