package com.sztzjy.fund_investment.controller.tea;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.fund_investment.annotation.AnonymousAccess;
import com.sztzjy.fund_investment.config.security.TokenProvider;
import com.sztzjy.fund_investment.entity.TeaClassScore;
import com.sztzjy.fund_investment.entity.TrainingReport;
import com.sztzjy.fund_investment.entity.UserTable;
import com.sztzjy.fund_investment.entity.UserTableExample;
import com.sztzjy.fund_investment.entity.dto.*;
import com.sztzjy.fund_investment.mapper.TrainingReportMapper;
import com.sztzjy.fund_investment.mapper.UserTableMapper;
import com.sztzjy.fund_investment.service.PerformanceScoreService;
import com.sztzjy.fund_investment.service.tea.ClassScoreService;
import com.sztzjy.fund_investment.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author xcj
 * @Date 2023/11/24
 */
@RestController
@RequestMapping("/tea/grade")
@Api(tags = "老师端--成绩管理&成绩详情")
public class ClassScoreController {

    @Autowired
    private ClassScoreService classScoreService;
    @Autowired
    private TrainingReportMapper trainingReportMapper;
    @Autowired
    private PerformanceScoreService performanceScoreService;
    @Autowired
    private UserTableMapper userTableMapper;


    @AnonymousAccess
    @GetMapping("/getClassAVGScore")
    @ApiOperation("成绩管理/班级平均成绩折线图")
    public ResultEntity<List<TeaClassScore>> getClassAVGScore(@RequestParam String schoolId) {
        return new ResultEntity<>(classScoreService.getClassAVGScore(schoolId));
    }

    @AnonymousAccess
    @GetMapping("/getClassScoreCount")
    @ApiOperation("成绩管理/班级成绩统计分析饼状图")
    public ResultEntity<TeaClassScoreDto> getClassAVGScore(@ApiParam("班级框为空时传") @RequestParam(required = false) String schoolId,
                                                           @ApiParam("班级框不为空时传") @RequestParam(required = false) String classId,
                                                           @ApiParam("年月日格式/yyyy-MM-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") @RequestParam Date time) {
        return new ResultEntity<>(classScoreService.getClassScoreCount(schoolId, classId, time));
    }

    @AnonymousAccess
    @GetMapping("/getStudentScoreDetails")
    @ApiOperation("成绩管理/成绩管理页面查询/展示")
    public ResultEntity<PageInfo<UserDto>> getStudentScoreDetails(@RequestParam Integer index,
                                                                  @RequestParam Integer size,
                                                                  @RequestParam String schoolId,
                                                                  @RequestParam(required = false) String classId,
                                                                  @RequestParam(required = false) String studentIdOrName) {
        return new ResultEntity<>(classScoreService.getStudentScoreDetails(index, size, schoolId, classId, studentIdOrName));
    }

    @AnonymousAccess
    @GetMapping("/importExcel")
    @ApiOperation("成绩管理/导出")
    public void importExcel(HttpServletResponse response, @RequestParam String schoolId) {
        classScoreService.importExcel(response, schoolId);
    }

    @AnonymousAccess
    @GetMapping("/autoScore")
    @ApiOperation("成绩管理/智能评分")
    public ResultEntity<HttpStatus> autoScore(@RequestParam String schoolId) {
        return classScoreService.autoScore(schoolId);
    }

    @AnonymousAccess
    @GetMapping("/getPerformanceScoreDto")
    @ApiOperation("成绩详情/实训进度页面/展示")
    public ResultEntity<PerformanceScoreDto> getPerformanceScoreDto(@ApiParam("页面返回的得分ID") @RequestParam String performanceScoreId) {
        return new ResultEntity<>(classScoreService.getPerformanceScoreDto(performanceScoreId));
    }

    @AnonymousAccess
    @GetMapping("/getStuTrainingScore")
    @ApiOperation("成绩详情&投资报告/实训成绩展示")
    public ResultEntity<PerformanceTrainingScoreDto> getStuTrainingScore(@ApiParam("页面返回的得分ID") @RequestParam String performanceScoreId) {
        return new ResultEntity<>(classScoreService.getStuTrainingScore(performanceScoreId));
    }

    @AnonymousAccess
    @GetMapping("/getReportStringByFlowId")
    @ApiOperation("成绩详情/实训成绩/五个文本框展示")
    public ResultEntity<FoundProjectDto> getReportStringByFlowId(@RequestParam String flowId) {
        return new ResultEntity<>(classScoreService.getReportStringByFlowId(flowId));
    }

    @AnonymousAccess
    @PostMapping("/updateScoreByFlowId")
    @ApiOperation("成绩详情/实训成绩/五个文本框修改分数")
    public ResultEntity<String> updateScoreByFlowId(@RequestParam String flowId,
                                                    @ApiParam("传Map的json字符串,例：{\"业务尽调结论\": 1,\"财务尽调结论\": 2}") @RequestParam String json) {
        Map<String, Integer> map;
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            map = objectMapper.readValue(json, new TypeReference<Map<String, Integer>>() {
            });
        } catch (Exception e) {
            throw new ClassCastException("类型转换错误");
        }
        return classScoreService.updateScoreByFlowId(flowId, map);
    }


    @AnonymousAccess
    @PostMapping("/getExperience")
    @ApiOperation("成绩详情/实训心得/页面展示")
    public ResultEntity<TrainingReport> getExperience(@RequestParam String reportId) {
        return new ResultEntity<TrainingReport>(trainingReportMapper.selectByPrimaryKey(reportId));
    }


    @AnonymousAccess
    @GetMapping("/downloadReport")
    @ApiOperation("成绩详情/实训心得/报告下载")
    public void downloadReport(@RequestParam String reportId, String TOKEN, HttpServletResponse response) {
        TokenProvider.getJWTUser(TOKEN);
        classScoreService.downloadReport(reportId, response);
    }


    @AnonymousAccess
    @PostMapping("/updateExperienceScore")
    @ApiOperation("成绩详情/实训心得/修改心得或报告分数")
    public void updateScoreByFlowId(@RequestParam String reportId,
                                    @RequestParam Integer score,
                                    @ApiParam("0为心得,1为报告") @RequestParam Integer type) {
        TrainingReport trainingReport = trainingReportMapper.selectByPrimaryKey(reportId);

        if (type == 0) {
            trainingReport.setExperienceScore(BigDecimal.valueOf(score));
            performanceScoreService.calculateScoreByModule("experienceScore", score, trainingReport.getFlowId());
        }
        if (type == 1) {
            trainingReport.setReportScore(BigDecimal.valueOf(score));
            performanceScoreService.calculateScoreByModule("investmentReportScore", score, trainingReport.getFlowId());
        }
        trainingReportMapper.updateByPrimaryKey(trainingReport);
    }

    @AnonymousAccess
    @GetMapping("/getClassNameBySchoolId")
    @ApiOperation("成绩管理&互动答疑/班级下拉框")
    public ResultEntity<List<String>> getClassNameBySchoolId(@RequestParam String schoolId) {
        UserTableExample userTableExample = new UserTableExample();
        userTableExample.createCriteria().andSchoolIdEqualTo(schoolId);
        List<UserTable> userTables = userTableMapper.selectByExample(userTableExample);
        List<String> list = new ArrayList<>();
        for (UserTable userTable : userTables) {
            list.add(userTable.getClassName());
        }
        return new ResultEntity<List<String>>(list);
    }


}
