package com.sztzjy.fund_investment.controller.tea;

import com.github.pagehelper.PageInfo;
import com.sztzjy.fund_investment.annotation.AnonymousAccess;
import com.sztzjy.fund_investment.entity.QuestionAnswerWithBLOBs;
import com.sztzjy.fund_investment.entity.TopicsWithBLOBs;
import com.sztzjy.fund_investment.entity.dto.QuestionAnswerDto;
import com.sztzjy.fund_investment.mapper.QuestionAnswerMapper;
import com.sztzjy.fund_investment.mapper.TopicsMapper;
import com.sztzjy.fund_investment.service.tea.TeaTopicManageService;
import com.sztzjy.fund_investment.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

/**
 * @Author xcj
 * @Date 2023/12/1
 */

@RequestMapping("/tea/topics")
@RestController
@Api(tags = "老师端--试题管理&互动答疑")
public class TeaTopicManageController {
    @Autowired
    private TeaTopicManageService teaTopicManageService;
    @Autowired
    private TopicsMapper topicsMapper;
    @Autowired
    private QuestionAnswerMapper questionAnswerMapper;


    @AnonymousAccess
    @PostMapping("/insertTopics")
    @ApiOperation("试题管理/新增试题")
    public ResultEntity<String> insertTopics(@ApiParam("topicId给字符串uuid,analysis,userAnswer,选项CDE允许为空,其他必填，" +
                                                        "source固定为【自增】，module值同导入功能模块,判断答案为A/B,多选为A,B,C英文逗号隔开")
                                             @RequestBody TopicsWithBLOBs topics) {
        return teaTopicManageService.insertTopics(topics);
    }


    @AnonymousAccess
    @GetMapping("/getTopicsBySchoolId")
    @ApiOperation("试题管理/页面搜索展示")
    public ResultEntity<PageInfo<TopicsWithBLOBs>> getTopicsBySchoolId(@RequestParam Integer index,
                                                                       @RequestParam Integer size,
                                                                       @RequestParam String schoolId,
                                                                       @ApiParam("module值同导入功能模块") @RequestParam String module,
                                                                       @ApiParam("搜索框/题干") @RequestParam(required = false) String topicContent) {
        return new ResultEntity<PageInfo<TopicsWithBLOBs>>(teaTopicManageService.getTopicsBySchoolId(index, size, schoolId, module, topicContent));
    }

    @AnonymousAccess
    @GetMapping("/getByTopicId")
    @ApiOperation("试题管理/查看单条数据")
    public ResultEntity<TopicsWithBLOBs> getByTopicId(@RequestParam String topicId) {
        return new ResultEntity<TopicsWithBLOBs>(topicsMapper.selectByPrimaryKey(topicId));
    }


    @AnonymousAccess
    @DeleteMapping("/deleteByTopicId")
    @ApiOperation("试题管理/删除单个试题(只能删除自主新增试题)")
    public ResultEntity<String> deleteByTopicId(@RequestParam String topicId) {
        TopicsWithBLOBs topicsWithBLOBs = topicsMapper.selectByPrimaryKey(topicId);
        if (topicsWithBLOBs.getSource().equals("内置")) {
            return new ResultEntity(HttpStatus.OK, "内置试题不支持删除！");
        }
        topicsMapper.deleteByPrimaryKey(topicId);
        return new ResultEntity(HttpStatus.OK, "删除成功！");
    }


    @AnonymousAccess
    @GetMapping("/getQuestionAnswer")
    @ApiOperation("互动答疑/展示搜索")
    public ResultEntity<PageInfo<QuestionAnswerDto>> getQuestionAnswer(@RequestParam Integer index,
                                                                       @RequestParam Integer size,
                                                                       @RequestParam String schoolId,
                                                                       @ApiParam("班级下拉框") @RequestParam(required = false) String className,
                                                                       @ApiParam("姓名/学号") @RequestParam(required = false) String keyWord) {
        return new ResultEntity<>(teaTopicManageService.getQuestionAnswer(index, size, schoolId, className, keyWord));
    }

    @AnonymousAccess
    @GetMapping("/recover")
    @ApiOperation("互动答疑/回复问题")
    public ResultEntity<String> deleteByTopicId(@ApiParam("展示接口中的id字段") @RequestParam String questionId,
                                                @ApiParam("回复内容") String content) {
        QuestionAnswerWithBLOBs questionAnswerWithBLOBs = questionAnswerMapper.selectByPrimaryKey(questionId);
        if (StringUtils.isBlank(content)){
            return new ResultEntity(HttpStatus.BAD_REQUEST, "请输入回复内容！");
        }
        questionAnswerWithBLOBs.setReplyContent(content);
        questionAnswerMapper.updateByPrimaryKey(questionAnswerWithBLOBs);
        return new ResultEntity(HttpStatus.OK, "回复成功！");
    }


    @AnonymousAccess
    @GetMapping("/getQuestionById")
    @ApiOperation("互动答疑/获取单挑互动数据")
    public ResultEntity<QuestionAnswerWithBLOBs> getQuestionById(@ApiParam("展示接口中的id字段") @RequestParam String questionId) {
        return new ResultEntity<QuestionAnswerWithBLOBs>(questionAnswerMapper.selectByPrimaryKey(questionId));
    }

}
