package com.sztzjy.fund_investment.service.serviceImpl;

import com.sztzjy.fund_investment.entity.TrainingReport;
import com.sztzjy.fund_investment.entity.TrainingReportExample;
import com.sztzjy.fund_investment.mapper.TrainingReportMapper;
import com.sztzjy.fund_investment.service.InvestmentReportService;
import com.sztzjy.fund_investment.util.ResultEntity;
import com.sztzjy.fund_investment.util.file.IFileUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.xmlbeans.impl.xb.xsdschema.Public;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * @Author xcj
 * @Date 2023/12/4
 */
@Service
public class InvestmentReportServiceImpl implements InvestmentReportService {
    @Autowired
    private IFileUtil fileUtil;
    @Autowired
    TrainingReportMapper trainingReportMapper;


    /**报告提交
     * @author xcj
     * @Date 2023/12/4
     */
    @Override
    public ResultEntity<String> uploadReport(MultipartFile file, String fileName, String flowId, String schoolId) {
        String filePath = fileUtil.upload(file);
        String originalFilename = file.getOriginalFilename();
        String fileExtension = originalFilename.substring(originalFilename.lastIndexOf(".")).toLowerCase();
        if (!fileExtension.equals(".pdf") && !fileExtension.equals(".doc") && !fileExtension.equals(".docx")) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "文件必须为word或pdf！");
        }
        List<TrainingReport> trainingReports = getTrainingReports(flowId);

        //只允许提交一次，判断是否只提交了心得没提交报告
        if (!trainingReports.isEmpty()) {
            TrainingReport trainingReport = trainingReports.get(0);
            if (StringUtils.isNotBlank(trainingReport.getReportName())) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "不允许重复提交！");
            } else {
                trainingReport.setUrl(filePath);
                trainingReport.setReportName(fileName);
                trainingReport.setUploadtime(new Date());
                trainingReport.setStep("投资报告");
                trainingReportMapper.updateByPrimaryKey(trainingReport);
                return new ResultEntity<>(HttpStatus.OK, "报告上传成功！");
            }
        } else {  //没上传报告没提交心得的情况
            TrainingReport trainingReport = new TrainingReport();
            trainingReport.setId(UUID.randomUUID().toString());
            trainingReport.setFlowId(flowId);
            trainingReport.setSchoolId(schoolId);
            trainingReport.setUrl(filePath);
            trainingReport.setReportName(fileName);
            trainingReport.setUploadtime(new Date());
            trainingReport.setStep("投资报告");
            trainingReportMapper.insert(trainingReport);
            return new ResultEntity<>(HttpStatus.OK, "报告上传成功！");
        }
    }


    @Override
    public List<TrainingReport> getTrainingReports(String flowId) {
        TrainingReportExample trainingReportExample = new TrainingReportExample();
        trainingReportExample.createCriteria().andFlowIdEqualTo(flowId);
        return trainingReportMapper.selectByExample(trainingReportExample);
    }


    /* 提交心得
     * @author xcj
     * @Date 2023/12/4
     */
    @Override
    public ResultEntity<String> commitExperience(TrainingReport trainingReport) {
        if (StringUtils.isBlank(trainingReport.getExperience())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "请先填写心得再提交！");
        }
        if (StringUtils.isBlank(trainingReport.getId()) || StringUtils.isBlank(trainingReport.getFlowId()) || StringUtils.isBlank(trainingReport.getSchoolId())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "提交失败！缺少所需的ID！");
        }
        List<TrainingReport> trainingReports = getTrainingReports(trainingReport.getFlowId());
        if (!trainingReports.isEmpty()) {
            TrainingReport dataTrainingReport = trainingReports.get(0);
            if (StringUtils.isNotBlank(dataTrainingReport.getExperience())){
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "不允许重复提交！");
            }else { //提交了报告没提交心得
                trainingReport.setExperience(trainingReport.getExperience());
                trainingReport.setUploadtime(new Date());
                trainingReportMapper.updateByPrimaryKey(trainingReport);
            }
        } else { //为空新增
            trainingReportMapper.insert(trainingReport);
        }
        return new ResultEntity<>(HttpStatus.OK, "提交成功！");
    }
}
