package com.sztzjy.fund_investment.service.serviceImpl;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.fund_investment.config.Constant;
import com.sztzjy.fund_investment.entity.*;
import com.sztzjy.fund_investment.mapper.PerformanceScoreMapper;
import com.sztzjy.fund_investment.mapper.TopicRecordMapper;
import com.sztzjy.fund_investment.mapper.TopicsMapper;
import com.sztzjy.fund_investment.service.PerformanceScoreService;
import com.sztzjy.fund_investment.service.TopicService;
import com.sztzjy.fund_investment.util.ResultEntity;
import org.apache.poi.ss.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.util.*;

/**
 * @Author xcj
 * @Date 2023/11/15
 */
@Service
public class TopicServiceImpl implements TopicService {
    @Autowired
    private TopicsMapper topicsMapper;
    @Autowired
    private PerformanceScoreService performanceScoreService;
    @Autowired
    private PerformanceScoreMapper performanceScoreMapper;
    @Autowired
    private TopicRecordMapper topicRecordMapper;

    private static final Logger log = LoggerFactory.getLogger(TopicServiceImpl.class);


    /* 导入功能
     * @author xcj
     * @Date 2023/11/22
     */
    public int importFile(MultipartFile multipartFile, String schoolId, String module) {
        File file = null;
        Workbook workbook = null;
        int totalNum = 0;
        /*得到的path是 /D:/springbootdemo/target/springbootdemo/WEB-INF/classes/  */
//        String path = TopicServiceImpl.class.getClassLoader().getResource("/").getPath();
//        /*拼接后的path就是 /D:/springbootdemo/target/springbootdemo/WEB-INF/电影正式资源.xlsx /*/
//        path = path.substring(0, path.indexOf("WEB-INF") + "WEB-INF".length()) + "/" + multipartFile.getOriginalFilename();
        file = new File(IdUtil.simpleUUID());
        try {
            /*把文件流copy读取到文件中*/
            FileCopyUtils.copy(multipartFile.getBytes(), file);
            workbook = WorkbookFactory.create(new FileInputStream(file));
            List<Topics> list = new ArrayList<>();
            /*遍历sheet页*/
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (sheet == null|| sheet.getLastRowNum()==0) {
                    continue;
                }
                /*统计导入的总条数，要是你的excell包含了表头，就不用加1了*/
                if (sheet.getLastRowNum() > 0) {
                    totalNum += sheet.getLastRowNum();
                }
                /*遍历行，这里j的初始值取1是因为我的表格里第一行是表头*/
                for (int j = 1; j < sheet.getPhysicalNumberOfRows(); j++) {
                    Row row = sheet.getRow(j);
                    /*解析列，下标从0开始*/
                    Cell cell0 = row.getCell(0);
                    Cell cell1 = row.getCell(1);
                    Cell cell2 = row.getCell(2);
                    Cell cell3 = row.getCell(3);
                    Cell cell4 = row.getCell(4);
                    Cell cell5 = row.getCell(5);
                    Cell cell6 = row.getCell(6);
                    Cell cell7 = row.getCell(7);
                    Cell cell8 = row.getCell(8);
                    Cell cell9 = row.getCell(9);
                    //判断题的BCDE选项和解析可能为空
                    if (cell0 == null || cell1 == null || cell2 == null || cell3 == null || cell4 == null || cell8 == null) {
                        continue;
                    }
                    String topicType = this.getCellValue(cell0);
                    String scorestr = this.getCellValue(cell1);
                    int score = Integer.parseInt(scorestr);
                    String content = this.getCellValue(cell2);
                    String choicesa = this.getCellValue(cell3);
                    String choicesb = this.getCellValue(cell4);
                    String choicesc = cell5 != null ? this.getCellValue(cell5) : null;
                    String choicesd = cell6 != null ? this.getCellValue(cell6) : null;
                    String choicese = cell7 != null ? this.getCellValue(cell7) : null;
                    String analysis = cell9 != null ? this.getCellValue(cell9) : null;
                    String answer = this.getCellValue(cell8);

                    /*我这里省略了很多数据清洗、校验的代码*/
                    TopicsWithBLOBs topics = new TopicsWithBLOBs();
                    topics.setTopicId(IdUtil.simpleUUID());
                    topics.setSource(Constant.ZZXZ);
                    topics.setModule(module);
                    topics.setTopicType(topicType);
                    topics.setScore(score);
                    topics.setTopicContent(content);
                    topics.setChoicesa(choicesa);
                    topics.setChoicesb(choicesb);
                    topics.setChoicesc(choicesc);
                    topics.setChoicesd(choicesd);
                    topics.setChoicese(choicese);
                    topics.setAnswer(answer);
                    topics.setAnalysis(analysis);
                    topics.setSchoolId(schoolId);
                    list.add(topics);
                }
                /*持久化：批量新增*/
                topicsMapper.insertBatch(list);
            }
            /*解析完删除此路径下的文件*/
            file.delete();
            return totalNum;
        } catch (Exception e) {
            e.printStackTrace();
            log.error("导入功能公用类异常exception={}", e);
        }
        return totalNum;
    }


    public String getCellValue(Cell cell) {
        if (cell.getCellType() == Cell.CELL_TYPE_BOOLEAN) {
            return String.valueOf(cell.getBooleanCellValue());
        } else if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            Double d = cell.getNumericCellValue();
            return String.valueOf(d.intValue());
        }
        return String.valueOf(cell.getStringCellValue());
    }


    /* 知识测评,项目估值,项目尽调,IPO申请,判断答案对错算分
     * @author xcj
     * @Date 2023/11/21
     */
    @Override
    public ResultEntity<List<String>> getRightScore(List<TopicsWithBLOBs> topics, String flowId) {
        int score = 0;
        List<String> list = new ArrayList<>();
        String module = "";
        for (TopicsWithBLOBs topic : topics) {
            String userAnswer = topic.getUserAnswer();
            String topicId = topic.getTopicId();
            module = topic.getModule();
            TopicsWithBLOBs topicsData = topicsMapper.selectByPrimaryKey(topicId);
            String rightAnswer = topicsData.getAnswer();
            String topicType = topicsData.getTopicType();
            if (topicType.equals(Constant.MULTIPLECHOICE)) {
                // 将用户选择的答案和正确答案转换为字符数组或集合
                char[] userAnswerArray = userAnswer.toCharArray();
                char[] rightAnswerArray = rightAnswer.toCharArray();

//                // 检查答案长度是否相等
//                if (userAnswerArray.length != rightAnswerArray.length) {
//                    // 答案长度不相等，答案错误跳过循环
//                    continue;
//                }

                // 检查每个选项是否同时存在于用户答案和正确答案中
                boolean isCorrect = true;
                for (char c : rightAnswerArray) {
                    if (!containsChar(userAnswerArray, c)) {
                        isCorrect = false;
//                        break;
                    }
                }
                if (isCorrect && topicId.equals("999")) {
                    score = 4;
                } else if (isCorrect) {
                    // 答案正确
                    score = score + 1; // 设置得分为1或其他适当的值
                }
            } else {
                if (userAnswer.equals(rightAnswer)) {
                    score = score + 1;
                }
            }
            //添加到答题记录表
            TopicRecord topicRecord = new TopicRecord();
            String recordId = IdUtil.simpleUUID();
            topicRecord.setId(recordId);
            topicRecord.setTopicId(topicId);
            topicRecord.setUserAnswer(userAnswer);
            topicRecord.setAnswer(rightAnswer);
            topicRecord.setFlowId(flowId);
            topicRecord.setTopicContent(topic.getTopicContent());
            topicRecord.setChoicesa(topicsData.getChoicesa());
            topicRecord.setChoicesb(topicsData.getChoicesb());
            String choicesc = topicsData.getChoicesc() != null ? topicsData.getChoicesc() : null;
            String choicesd = topicsData.getChoicesd() != null ? topicsData.getChoicesd() : null;
            String choicese = topicsData.getChoicese() != null ? topicsData.getChoicese() : null;
            topicRecord.setChoicesc(choicesc);
            topicRecord.setChoicesd(choicesd);
            topicRecord.setChoicese(choicese);
            topicRecord.setModule(module);
            topicRecord.setTopicType(topicType);
            topicRecordMapper.insert(topicRecord);
            list.add(recordId);
        }
        PerformanceScore performanceScore = performanceScoreService.getByFlowId(flowId);

        //IPO模块总分十分，其余5分
        if (!(module.equals(Constant.IPOBK) || module.equals(Constant.IPOTJ)) && score >= 5) {
            score = 5;
        }
//      调用计分方法
        if (module.equals(Constant.ZSCETK)) {
            performanceScoreService.calculateScoreByModule("knowledgeAssessmentScore", score, flowId);
            //标记完成时间
            performanceScore.setKnowledgeAssessmentTime(new Date());
            performanceScoreMapper.updateByPrimaryKey(performanceScore);
        }
        if (module.equals(Constant.XMJDTK)) {
            performanceScoreService.calculateScoreByModule("projectDueDiligenceScore", score, flowId);
        }
        if (module.equals(Constant.XMGZTK)) {
            performanceScoreService.calculateScoreByModule("projectValuationScore", score, flowId);
        }

        //todo 暂时调用一个，两个字段是否合成一个后面再看
        if (module.equals(Constant.IPOBK) || module.equals(Constant.IPOTJ)) {
            performanceScoreService.calculateScoreByModule("ipoListedSectorScore", score, flowId);
        }
        return new ResultEntity<>(list);
    }


    /* 检查字符数组是否包含指定字符
     * @author xcj
     * @Date 2023/11/21
     */
    private boolean containsChar(char[] array, char c) {
        for (char element : array) {
            if (element == c) {
                return true;
            }
        }
        return false;
    }


    /*
     * @author xcj
     * @Date 2023/11/21
     */
//    @Override
//    public List<Topics> getRandomTopic(String module) {
//        TopicsExample topicsExample = new TopicsExample();
//        topicsExample.createCriteria().andModuleEqualTo(module);
//        List<Topics> topics = topicsMapper.selectByExampleWithBLOBs(topicsExample);
//        //随机取6道题目
//        Collections.shuffle(topics);
//        int size = Math.min(6, topics.size());
//        List<Topics> randomTopics = topics.subList(0, size);
//        List<Topics> list = new ArrayList<>();
//        for (Topics randomTopic : randomTopics) {
//            randomTopic.setAnswer(null);
//            TopicsDto topicsDto = new TopicsDto();
//            BeanUtils.copyProperties(randomTopic, topicsDto);
//            String choices = randomTopic.getChoices(); //选项
//            Map<String, String> map = getStringToMap(choices);
//            topicsDto.setChoices(map);
//            list.add(randomTopic);
//        }
//        return list;
//    }

    @Override
    public List<TopicsWithBLOBs> getRandomTopic(String module) {
        TopicsExample topicsExample = new TopicsExample();
        topicsExample.createCriteria().andModuleEqualTo(module);
        List<TopicsWithBLOBs> topics = topicsMapper.selectByExampleWithBLOBs(topicsExample);
        //随机取6道题目
        Collections.shuffle(topics);
        int size = Math.min(6, topics.size());
        List<TopicsWithBLOBs> randomTopics = topics.subList(0, size);
        for (Topics randomTopic : randomTopics) {
            randomTopic.setAnswer(null);
        }
        return randomTopics;
    }




    /* 回显答题记录
     * @author xcj
     * @Date 2023/11/21
     */
    @Override
    public List<TopicRecord> getTopicRecord(List<String> ids) {
        List<TopicRecord> list =new ArrayList<>();
        for (String id : ids) {
            TopicRecord topicRecord = topicRecordMapper.selectByPrimaryKey(id);
            list.add(topicRecord);
        }
        return list;
    }



    /* IPO页面弹窗2使用返回随机6条数据
     * @author xcj
     * @Date 2023/11/21
     */
    @Override
    public List<TopicsWithBLOBs> getIPOTopicTwo() {
        int topicCountPerModule = 3;
        List<TopicsWithBLOBs> topics = getRandomTopicsByModule(Constant.IPOBK, topicCountPerModule);
        List<TopicsWithBLOBs> topics1 = getRandomTopicsByModule(Constant.IPOTJ, topicCountPerModule);
        topics.addAll(topics1);
//        List<TopicsDto> list = toTopicsDtoList(topics);
//        list.addAll(toTopicsDtoList(topics1));
        return topics;
    }


    /* IPO两个模块分别取三题
     * @author xcj
     * @Date 2023/11/21
     */
    private List<TopicsWithBLOBs> getRandomTopicsByModule(String module, int count) {
        TopicsExample topicsExample = new TopicsExample();
        topicsExample.createCriteria().andModuleEqualTo(module);
        List<TopicsWithBLOBs> topics = topicsMapper.selectByExampleWithBLOBs(topicsExample);
        Collections.shuffle(topics);
        int size = Math.min(count, topics.size());
        return topics.subList(0, size);
    }


    /* 选项转Map，对象转Dto返回
     * @author xcj
     * @Date 2023/11/21
     */
//    private List<TopicsDto> toTopicsDtoList(List<Topics> topics) {
//        List<TopicsDto> list = new ArrayList<>();
//        for (Topics topic : topics) {
//            topic.setAnswer(null);
//            TopicsDto topicsDto = new TopicsDto();
//            BeanUtils.copyProperties(topic, topicsDto);
//            String choices = topic.getChoices();
//            Map<String, String> map = getStringToMap(choices);
//            topicsDto.setChoices(map);
//            list.add(topicsDto);
//        }
//        return list;
//    }


    /* 字符串转map
     * @author xcj
     * @Date 2023/11/21
     */
    private Map<String, String> getStringToMap(String choices) {
        String[] split = choices.split(",");
        Map<String, String> map = new HashMap<>();
        for (String pair : split) {
            String[] keyValue = pair.split("\\.");
            if (keyValue.length == 2) {
                String key = keyValue[0];
                String value = keyValue[1];
                map.put(key, value);
            }
        }
        return map;
    }



    public static void main(String[] args) throws Exception {
        //字符串切割转list
        String listString = "A:111,B:2222,C:333";
        listString = listString.replace(",", "");
        String[] stringArray = listString.split(",");
        List<String> stringList = Arrays.asList(stringArray);
        System.out.println(stringList);

    }
}
