package com.sztzjy.fund_investment.service.serviceImpl.tea;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageInfo;
import com.sztzjy.fund_investment.config.Constant;
import com.sztzjy.fund_investment.entity.*;
import com.sztzjy.fund_investment.entity.dto.*;
import com.sztzjy.fund_investment.mapper.*;
import com.sztzjy.fund_investment.service.PerformanceScoreService;
import com.sztzjy.fund_investment.service.tea.ClassScoreService;
import com.sztzjy.fund_investment.util.PageUtil;
import com.sztzjy.fund_investment.util.ResultEntity;
import com.sztzjy.fund_investment.util.excel.FilePortUtil;
import com.sztzjy.fund_investment.util.file.IFileUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * @Author xcj
 * @Date 2023/11/24
 */
@Service
public class ClassScoreServiceImpl implements ClassScoreService {
    @Autowired
    private TeaClassScoreMapper teaClassScoreMapper;
    @Autowired
    private UserTableMapper userMapper;
    @Autowired
    private TrainingReportMapper reportMapper;
    @Autowired
    private FlowMapper flowMapper;
    @Autowired
    private PerformanceScoreMapper performanceScoreMapper;
    @Autowired
    private FoundProjectMapper foundProjectMapper;
    @Autowired
    private PerformanceScoreService performanceScoreService;
    @Autowired
    private TrainingReportMapper trainingReportMapper;
    @Autowired
    private IFileUtil fileUtil;


    /*
     * @author xcj
     * @Date 2023/11/24
     */
    @Override
    public List<TeaClassScore> getClassAVGScore(String schoolId) {
        return teaClassScoreMapper.selectAndOrderByTime(schoolId);
    }


    /*
     * @author xcj
     * @Date 2023/11/24
     */
    @Override
    public TeaClassScoreDto getClassScoreCount(String schoolId, String classId, Date time) {
        TeaClassScoreExample teaClassScoreExample = new TeaClassScoreExample();
        TeaClassScoreExample.Criteria criteria = teaClassScoreExample.createCriteria();
        //班级框为空 统计学校下的所有数据返回
        if (classId == null && schoolId != null) {
            criteria.andSchoolIdEqualTo(schoolId).andStartTimeEqualTo(time);
            List<TeaClassScore> teaClassScores = teaClassScoreMapper.selectByExample(teaClassScoreExample);
            int schoolExcellentCount = 0;
            int schoolGoodCount = 0;
            int schoolGeneralCount = 0;
            int schoolFailCount = 0;
            BigDecimal maxScoreBySchoolId = BigDecimal.ZERO;
            BigDecimal minScoreBySchoolId = null;
            BigDecimal avgScoreBySchoolId = BigDecimal.ZERO;
            for (TeaClassScore teaClassScore : teaClassScores) {
                schoolExcellentCount = schoolExcellentCount + teaClassScore.getExcellentCount();
                schoolGoodCount = schoolGoodCount + teaClassScore.getGoodCount();
                schoolGeneralCount = schoolGeneralCount + teaClassScore.getGeneralCount();
                schoolFailCount = schoolFailCount + teaClassScore.getFailCount();

                BigDecimal maxScore = teaClassScore.getClassMaxScore();
                BigDecimal minScore = teaClassScore.getClassMinScore();
                // 计算最高分
                if (maxScore.compareTo(maxScoreBySchoolId) >= 0) {
                    maxScoreBySchoolId = maxScore;
                }
                // 计算最低分
                if (minScoreBySchoolId == null || minScore.compareTo(minScoreBySchoolId) <= 0) {
                    minScoreBySchoolId = minScore;
                }
                //所有班级平均分累加
                avgScoreBySchoolId = avgScoreBySchoolId.add(teaClassScore.getClassAverageScore()).setScale(2, RoundingMode.HALF_UP);
            }
            TeaClassScoreDto teaClassScoreDto = new TeaClassScoreDto();
            teaClassScoreDto.setSchoolExcellentCount(schoolExcellentCount);
            teaClassScoreDto.setSchoolGoodCount(schoolGoodCount);
            teaClassScoreDto.setSchoolGeneralCount(schoolGeneralCount);
            teaClassScoreDto.setSchoolFailCount(schoolFailCount);
            BigDecimal finalAVGScore = avgScoreBySchoolId.divide(BigDecimal.valueOf(teaClassScores.size()), 2, RoundingMode.HALF_UP);
            teaClassScoreDto.setClassAverageScore(finalAVGScore);
            return teaClassScoreDto;
        } else {        //选中某个班级返回
            criteria.andClassIdEqualTo(classId).andStartTimeEqualTo(time);
            List<TeaClassScore> teaClassScores = teaClassScoreMapper.selectByExample(teaClassScoreExample);
            TeaClassScore teaClassScore = teaClassScores.get(0);
            TeaClassScoreDto teaClassScoreDto = new TeaClassScoreDto();
            BeanUtils.copyProperties(teaClassScore, teaClassScoreDto);
            return teaClassScoreDto;
        }
    }


    /* 成绩详情页面
     * @author xcj
     * @Date 2023/11/27
     */
    @Override
    public PageInfo<UserDto> getStudentScoreDetails(Integer index, Integer size, String schoolId, String classId, String studentIdOrName) {
        UserTableExample userTableExample = new UserTableExample();
        UserTableExample.Criteria criteria = userTableExample.createCriteria();
        UserTableExample.Criteria orCriteria = userTableExample.createCriteria();
        if (StringUtils.isNotBlank(classId)) {
            criteria.andClassIdEqualTo(classId);
        }else {
            criteria.andSchoolIdEqualTo(schoolId);
        }
        if (StringUtils.isNotBlank(studentIdOrName)) {
            criteria.andNameEqualTo(studentIdOrName);
            orCriteria.andStudentIdEqualTo(studentIdOrName);
        }
        userTableExample.or(orCriteria);
        List<UserTable> userTables = userMapper.selectByExample(userTableExample);
        List<UserDto> userDtoList = new ArrayList<>();
        for (UserTable userTable : userTables) {
            UserDto userDto = new UserDto();
            BeanUtils.copyProperties(userTable, userDto);
            Flow flow = flowMapper.selectByPrimaryKey(userTable.getUserid());
            String flowId = flow.getFlowId();
            TrainingReportExample trainingReportExample = new TrainingReportExample();
            trainingReportExample.createCriteria().andFlowIdEqualTo(flowId); //todo 查询时看需不需要加上模块
            List<TrainingReport> trainingReports = reportMapper.selectByExample(trainingReportExample);
            PerformanceScoreExample performanceScoreExample = new PerformanceScoreExample();
            performanceScoreExample.createCriteria().andFlowIdEqualTo(flowId);
            List<PerformanceScore> performanceScores = performanceScoreMapper.selectByExample(performanceScoreExample);
            if (!performanceScores.isEmpty()) {
                PerformanceScore performanceScore = performanceScores.get(0);
                if (!trainingReports.isEmpty()) {
                    for (TrainingReport trainingReport : trainingReports) {
                        userDto.setReportId(trainingReport.getId());
                        userDto.setReportName(trainingReport.getReportName());
                    }
                    userDto.setFlowId(flowId);
                    userDto.setScore(performanceScore.getTotalScore());
                    userDto.setPerformanceScoreId(performanceScore.getId());
                    userDtoList.add(userDto);
                }
            }
        }
        if (userDtoList.size() <= 0) {
            return null;
        }
        userDtoList.sort(new StuUserDtoComparator()); //比较器按得分排序
        return PageUtil.pageHelper(userDtoList, index, size); //分页
    }


    /* 成绩详情导出功能
     * @author xcj
     * @Date 2023/11/27
     */
    @Override
    public void importExcel(HttpServletResponse response, String schoolId) {
        UserTableExample userTableExample = new UserTableExample();
        userTableExample.createCriteria().andSchoolIdEqualTo(schoolId);
        List<UserTable> userTables = userMapper.selectByExample(userTableExample);
        List<UserDto> userDtoList = new ArrayList<>();
        for (UserTable userTable : userTables) {
            UserDto userDto = new UserDto();
            BeanUtils.copyProperties(userTable, userDto);
            Flow flow = flowMapper.selectByPrimaryKey(userTable.getUserid());
            String flowId = flow.getFlowId();
            TrainingReportExample trainingReportExample = new TrainingReportExample();
            trainingReportExample.createCriteria().andFlowIdEqualTo(flowId).andSchoolIdEqualTo(schoolId);
            List<TrainingReport> trainingReports = reportMapper.selectByExample(trainingReportExample);
            PerformanceScoreExample performanceScoreExample = new PerformanceScoreExample();
            performanceScoreExample.createCriteria().andFlowIdEqualTo(flowId).andSchoolIdEqualTo(schoolId);
            List<PerformanceScore> performanceScores = performanceScoreMapper.selectByExample(performanceScoreExample);
            if (performanceScores.isEmpty()) {
                continue;
            }
            if (trainingReports.isEmpty()) {
                continue;
            }
            PerformanceScore performanceScore = performanceScores.get(0);
            for (TrainingReport trainingReport : trainingReports) {
                userDto.setReportName(trainingReport.getReportName());
            }
            userDto.setFlowId(flowId);
            userDto.setScore(performanceScore.getTotalScore());
            userDtoList.add(userDto);
        }
        userDtoList.sort(new StuUserDtoComparator()); //比较器按得分排序

        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"班级", "学号", "姓名", "成绩", "附件"};

        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("className", "studentId", "name", "score", "reportName");
        try {
            FilePortUtil.exportExcel(response, title, headers, userDtoList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /* 智能评分
     * @author xcj
     * @Date 2023/11/28
     */
    @Override
    public ResultEntity<HttpStatus> autoScore(String schoolId) {
        //根据flowId查出所有页面的结论报告
        PerformanceScoreExample performanceScoreExample = new PerformanceScoreExample();
        performanceScoreExample.createCriteria().andSchoolIdEqualTo(schoolId);
        List<PerformanceScore> performanceScores = performanceScoreMapper.selectByExample(performanceScoreExample);
        if (!performanceScores.isEmpty()) {
            for (PerformanceScore performanceScore : performanceScores) {
                String flowId = performanceScore.getFlowId();
                FoundProject foundProject = foundProjectMapper.selectByPrimaryKey(flowId);
                String foundProjectConclusion = foundProject.getFoundProjectConclusion(); // 寻找项目结论
                String serviceDueDiligence = foundProject.getServiceDueDiligence();  //  业务尽调结论
                String financialDueDiligence = foundProject.getFinancialDueDiligence(); //财务尽调结论
                String projectValuationRelative = foundProject.getProjectValuationRelative();//相对估值结论得分
                String projectValuationAbsolute = foundProject.getProjectValuationAbsolute();//绝对估值结论得分

                foundProjectConclusion = foundProjectConclusion.replaceAll("[^a-zA-Z0-9]", "");
                serviceDueDiligence = serviceDueDiligence.replaceAll("[^a-zA-Z0-9]", "");
                financialDueDiligence = financialDueDiligence.replaceAll("[^a-zA-Z0-9]", "");
                projectValuationRelative = projectValuationRelative.replaceAll("[^a-zA-Z0-9]", "");
                projectValuationAbsolute = projectValuationAbsolute.replaceAll("[^a-zA-Z0-9]", "");
                PerformanceScoreExample performanceScoreExample2 = new PerformanceScoreExample();
                performanceScoreExample2.createCriteria().andSchoolIdEqualTo(schoolId).andFlowIdEqualTo(flowId);
                // 寻找项目结论
                if (foundProjectConclusion.length() >= 30) {
                    // 字符串长度超过30
                    performanceScoreService.calculateScoreByModule("projectSearchReportScore", 2, flowId);
                } else {
                    //未超过30，结论报告是必填项，到这里就给1分没有0
                    performanceScoreService.calculateScoreByModule("projectSearchReportScore", 1, flowId);
                }

                //  业务尽调结论
                if (serviceDueDiligence.length() >= 30) {
                    performanceScoreService.calculateScoreByModule("projectDueDiligenceBusinessReportScore", 2, flowId);
                } else {
                    performanceScoreService.calculateScoreByModule("projectDueDiligenceBusinessReportScore", 1, flowId);
                }

                //财务尽调结论
                if (financialDueDiligence.length() >= 30) {
                    performanceScoreService.calculateScoreByModule("projectDueDiligenceFinanceReportScore", 2, flowId);
                } else {
                    performanceScoreService.calculateScoreByModule("projectDueDiligenceFinanceReportScore", 1, flowId);
                }

                //项目相对估值结论
                if (projectValuationRelative.length() >= 30) {
                    performanceScoreService.calculateScoreByModule("projectValuationRelativeScore", 2, flowId);
                } else {
                    performanceScoreService.calculateScoreByModule("projectValuationRelativeScore", 1, flowId);
                }
                //项目绝对估值结论
                if (projectValuationAbsolute.length() >= 30) {
                    performanceScoreService.calculateScoreByModule("projectValuationAbsoluteScore", 2, flowId);
                } else {
                    performanceScoreService.calculateScoreByModule("projectValuationAbsoluteScore", 1, flowId);
                }
                TrainingReportExample trainingReportExample = new TrainingReportExample();
                trainingReportExample.createCriteria().andSchoolIdEqualTo(schoolId).andFlowIdEqualTo(flowId);
                List<TrainingReport> trainingReports = trainingReportMapper.selectByExample(trainingReportExample);
                if (!trainingReports.isEmpty()) {
                    TrainingReport trainingReport = trainingReports.get(0);
                    String experience = trainingReport.getExperience();
                    String url = trainingReport.getUrl();
                    String reportName = trainingReport.getReportName();
                    experience = experience.replaceAll("[^a-zA-Z0-9]", "");
                    url = url.replaceAll("[^a-zA-Z0-9]", "");
                    reportName = reportName.replaceAll("[^a-zA-Z0-9]", "");
                    //实训心得
                    if (experience.length() < 100) {
                        performanceScoreService.calculateScoreByModule("projectValuationRelativeScore", 1, flowId); //todo 需要修改
                        trainingReport.setExperienceScore(BigDecimal.ONE);
                    }
                    if (experience.length() >= 100 && experience.length() < 300) {
                        performanceScoreService.calculateScoreByModule("projectValuationRelativeScore", 2, flowId);//todo 需要修改
                        trainingReport.setExperienceScore(BigDecimal.valueOf(2));
                    }
                    if (experience.length() >= 300) {
                        performanceScoreService.calculateScoreByModule("projectValuationRelativeScore", 3, flowId);//todo 需要修改
                        trainingReport.setExperienceScore(BigDecimal.valueOf(3));
                    }

                    //实验报告，未上传 0 分 上传得两分
                    if (!(url.isEmpty()) && !(reportName.isEmpty())) {
                        performanceScoreService.calculateScoreByModule("investmentReportScore", 2, flowId);//todo 需要修改
                        trainingReport.setReportScore(BigDecimal.valueOf(2));
                    }
                    trainingReportMapper.updateByPrimaryKey(trainingReport);
                }
            }
            return new ResultEntity<>(HttpStatus.OK);
        }
        return new ResultEntity<>(HttpStatus.NOT_FOUND);
    }

    /* 用户成绩详情展示
     * @author xcj
     * @Date 2023/11/29
     */
    @Override
    public PerformanceScoreDto getPerformanceScoreDto(String performanceScoreId) {
        PerformanceScore performanceScore = performanceScoreMapper.selectByPrimaryKey(performanceScoreId);
        PerformanceScoreDto performanceScoreDto = new PerformanceScoreDto();
        //实务认知
        if (performanceScore.getPracticalCognitionTime() != null) {
            performanceScoreDto.setPracticalCognitionSchedule(Constant.FINISHED);
            performanceScoreDto.setPracticalCognitionTime(performanceScore.getPracticalCognitionTime());
        } else {
            performanceScoreDto.setPracticalCognitionSchedule(Constant.UNFINISHED);
        }
        //知识测评
        if (performanceScore.getKnowledgeAssessmentTime() != null) {
            performanceScoreDto.setKnowledgeAssessmentSchedule(Constant.FINISHED);
            performanceScoreDto.setKnowledgeAssessmentTime(performanceScore.getKnowledgeAssessmentTime());
        } else {
            performanceScoreDto.setKnowledgeAssessmentSchedule(Constant.UNFINISHED);
        }
        //基金募资
        if (performanceScore.getFundraisingTime() != null) {
            performanceScoreDto.setFundraisingSchedule(Constant.FINISHED);
            performanceScoreDto.setFundraisingTime(performanceScore.getFundraisingTime());
        } else {
            performanceScoreDto.setFundraisingSchedule(Constant.UNFINISHED);
        }
        //寻找项目
        if (performanceScore.getProjectSearchTime() != null) {
            performanceScoreDto.setProjectSearchSchedule(Constant.FINISHED);
            performanceScoreDto.setProjectSearchTime(performanceScore.getProjectSearchTime());
        } else {
            performanceScoreDto.setProjectSearchSchedule(Constant.UNFINISHED);
        }
        //项目尽调
        if (performanceScore.getProjectDueDiligenceTime() != null) {
            performanceScoreDto.setProjectDueDiligenceSchedule(Constant.FINISHED);
            performanceScoreDto.setProjectDueDiligenceTime(performanceScore.getProjectDueDiligenceTime());
        } else {
            performanceScoreDto.setProjectDueDiligenceSchedule(Constant.UNFINISHED);
        }
        //项目估值
        if (performanceScore.getProjectValuationTime() != null) {
            performanceScoreDto.setProjectValuationSchedule(Constant.FINISHED);
            performanceScoreDto.setProjectValuationTime(performanceScore.getProjectValuationTime());
        } else {
            performanceScoreDto.setProjectValuationSchedule(Constant.UNFINISHED);
        }
        //签约投资
        if (performanceScore.getInvestmentSigningTime() != null) {
            performanceScoreDto.setInvestmentSigningSchedule(Constant.FINISHED);
            performanceScoreDto.setInvestmentSigningTime(performanceScore.getInvestmentSigningTime());
        } else {
            performanceScoreDto.setInvestmentSigningSchedule(Constant.UNFINISHED);
        }
        //IPO申请
        if (performanceScore.getIpoTime() != null) {
            performanceScoreDto.setIpoSchedule(Constant.FINISHED);
            performanceScoreDto.setIpoTime(performanceScore.getIpoTime());
        } else {
            performanceScoreDto.setIpoSchedule(Constant.UNFINISHED);
        }
        //询价发行
        if (performanceScore.getPricingIssuanceTime() != null) {
            performanceScoreDto.setPricingIssuanceSchedule(Constant.FINISHED);
            performanceScoreDto.setPricingIssuanceTime(performanceScore.getPricingIssuanceTime());
        } else {
            performanceScoreDto.setPricingIssuanceSchedule(Constant.UNFINISHED);
        }
        //收益管理
        if (performanceScore.getProfitManagementTime() != null) {
            performanceScoreDto.setProfitManagementSchedule(Constant.FINISHED);
            performanceScoreDto.setProfitManagementTime(performanceScore.getProfitManagementTime());
        } else {
            performanceScoreDto.setProfitManagementSchedule(Constant.UNFINISHED);
        }
        //收益分配
        if (performanceScore.getProfitDistributionTime() != null) {
            performanceScoreDto.setProfitDistributionSchedule(Constant.FINISHED);
            performanceScoreDto.setProfitDistributionTime(performanceScore.getProfitDistributionTime());
        } else {
            performanceScoreDto.setProfitDistributionSchedule(Constant.UNFINISHED);
        }
        //投资报告
        if (performanceScore.getInvestmentReportTime() != null) {
            performanceScoreDto.setInvestmentReportSchedule(Constant.FINISHED);
            performanceScoreDto.setInvestmentReportTime(performanceScore.getInvestmentReportTime());
        } else {
            performanceScoreDto.setInvestmentReportSchedule(Constant.UNFINISHED);
        }
        return performanceScoreDto;
    }


    /* 实训成绩展示
     * @author xcj
     * @Date 2023/11/29
     */
    @Override
    public PerformanceTrainingScoreDto getStuTrainingScore(String performanceScoreId) {
        PerformanceScore performanceScore = performanceScoreMapper.selectByPrimaryKey(performanceScoreId);
        PerformanceTrainingScoreDto performanceTrainingScoreDto = new PerformanceTrainingScoreDto();
        performanceTrainingScoreDto.setFlowId(performanceScore.getFlowId());
        performanceTrainingScoreDto.setPracticalCognitionScore(performanceScore.getPracticalCognitionScore());
        performanceTrainingScoreDto.setKnowledgeAssessmentScore(performanceScore.getKnowledgeAssessmentScore());
        performanceTrainingScoreDto.setFundraisingScore(performanceScore.getFundraisingScore());
        performanceTrainingScoreDto.setProjectSearchScore(performanceScore.getProjectSearchScore().add(performanceScore.getProjectSearchReportScore()));
        performanceTrainingScoreDto.setProjectDueDiligenceScore(performanceScore.getProjectDueDiligenceScore()
                .add(performanceScore.getProjectDueDiligenceFinanceReportScore()
                        .add(performanceScore.getProjectDueDiligenceBusinessReportScore()
                                .add(performanceScore.getProjectValuationEstimationRiskScore()))));

        performanceTrainingScoreDto.setProjectValuationScore(performanceScore.getProjectValuationRelativeScore()
                .add(performanceScore.getProjectValuationScore()
                        .add(performanceScore.getProjectValuationAbsoluteScore()
                                .add(performanceScore.getProjectUseValuationAbsoluteScore()
                                        .add(performanceScore.getProjectUseValuationRelativeScore())))));
        performanceTrainingScoreDto.setInvestmentSigningScore(performanceScore.getInvestmentSigningVlauationReportScore()
                .add(performanceScore.getInvestmentSigningDiligenceReportScore()
                        .add(performanceScore.getInvestmentSigningInvestScore()
                                .add(performanceScore.getInvestmentSigningScore()))));

        performanceTrainingScoreDto.setIpoScore(performanceScore.getIpoFlowScore().add(performanceScore.getIpoListedConditionScore()
                .add(performanceScore.getIpoListedSectorScore())));

        performanceTrainingScoreDto.setPricingIssuanceScore(performanceScore.getNewPricingIssuanceScore()
                .add(performanceScore.getPricingIssuanceConfirmScore().add(performanceScore.getInitPricingIssuanceScore()
                        .add(performanceScore.getPricingIssuanceEnteringScore()))));

        performanceTrainingScoreDto.setProfitManagementScore(performanceScore.getProfitManagementMarketValueScore()
                .add(performanceScore.getProfitManagementFundScore()
                        .add(performanceScore.getProfitManagementFundExitScore()
                                .add(performanceScore.getProfitManagementMarketValueExitScore()))));

        performanceTrainingScoreDto.setProfitDistributionScore(performanceScore.getProfitDistributionScore());
        performanceTrainingScoreDto.setInvestmentReportScore(performanceScore.getInvestmentReportScore().add(performanceScore.getExperienceScore()));
        performanceTrainingScoreDto.setTotalScore(performanceScore.getTotalScore());
        return performanceTrainingScoreDto;
    }

    /* 实训成绩页面展示五个结论及分数
     * @author xcj
     * @Date 2023/11/30
     */
    @Override
    public FoundProjectDto getReportStringByFlowId(String flowId) {
        FoundProject foundProject = foundProjectMapper.selectByPrimaryKey(flowId);
        FoundProjectDto dto = new FoundProjectDto();
        BeanUtils.copyProperties(foundProject, dto);
        PerformanceScoreExample example = new PerformanceScoreExample();
        example.createCriteria().andFlowIdEqualTo(flowId);
        List<PerformanceScore> performanceScores = performanceScoreMapper.selectByExample(example);
        PerformanceScore performanceScore = performanceScores.get(0);
        dto.setFoundProjectConclusionScore(performanceScore.getProjectSearchReportScore());
        dto.setServiceDueDiligenceScore(performanceScore.getProjectDueDiligenceBusinessReportScore());
        dto.setFinancialDueDiligenceScore(performanceScore.getProjectDueDiligenceFinanceReportScore());
        dto.setProjectValuationRelativeScore(performanceScore.getProjectValuationRelativeScore());
        dto.setProjectValuationAbsoluteScore(performanceScore.getProjectValuationAbsoluteScore());
        return dto;
    }


    /* 实训成绩修改五个文本框的分数
     * @author xcj
     * @Date 2023/11/30
     */
    @Override
    public ResultEntity<String> updateScoreByFlowId(String flowId, Map<String, Integer> map) {
        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            String key = entry.getKey();
            Integer score = entry.getValue();
            try {
                if (key.equals(Constant.XMGZFX)) { //寻找项目/评估结论得分
                    performanceScoreService.calculateScoreByModule("projectSearchReportScore", score, flowId);
                }
                if (key.equals(Constant.YWJDJL)) { //项目尽调/业务尽调结论得分
                    performanceScoreService.calculateScoreByModule("projectDueDiligenceBusinessReportScore", score, flowId);
                }
                if (key.equals(Constant.CWJDJL)) { //项目尽调/财务尽调结论得分
                    performanceScoreService.calculateScoreByModule("projectDueDiligenceFinanceReportScore", score, flowId);
                }
                if (key.equals(Constant.XDGZJL)) { //项目估值/相对估值结论得分
                    performanceScoreService.calculateScoreByModule("projectValuationRelativeScore", score, flowId);
                }
                if (key.equals(Constant.JDGZJL)) { //项目估值/绝对估值结论得分
                    performanceScoreService.calculateScoreByModule("projectValuationAbsoluteScore", score, flowId);
                }
            } catch (Exception e) {
                return new ResultEntity<>(HttpStatus.INTERNAL_SERVER_ERROR, "评分失败！");
            }
        }
        return new ResultEntity<>(HttpStatus.OK, "评分成功！");
    }

    /* 报告下载
     * @author xcj
     * @Date 2023/12/1
     */
    @Override
    public void downloadReport(String reportId, HttpServletResponse response) {
        TrainingReport trainingReport = trainingReportMapper.selectByPrimaryKey(reportId);
        Assert.isTrue(trainingReport != null && trainingReport.getUrl() != null, "报告不存在");
        fileUtil.download(response, trainingReport.getReportName(), trainingReport.getUrl());
    }


    /* 按得分比较排序
     * @author xcj
     * @Date 2023/11/6
     */
    public class StuUserDtoComparator implements Comparator<UserDto> {
        @Override
        public int compare(UserDto user1, UserDto user2) {
            // 根据评分进行比较
            if (user1.getScore() != null && user2.getScore() != null) {
                int ratingComparison = user2.getScore().compareTo(user1.getScore());
                if (ratingComparison != 0) {
                    return ratingComparison;
                }
            } else if (user1.getScore() != null) {
                return -1; // user2 的评分为 null，将 user1 排在前面
            } else if (user2.getScore() != null) {
                return 1; // user1 的评分为 null，将 user2 排在前面
            }
            return 0; // 如果评分相同，返回0，表示相等
        }
    }

}
