package com.sztzjy.fund_investment.service.serviceImpl.tea;

import com.github.pagehelper.PageInfo;
import com.sztzjy.fund_investment.entity.*;
import com.sztzjy.fund_investment.entity.dto.QuestionAnswerDto;
import com.sztzjy.fund_investment.mapper.FlowMapper;
import com.sztzjy.fund_investment.mapper.QuestionAnswerMapper;
import com.sztzjy.fund_investment.mapper.TopicsMapper;
import com.sztzjy.fund_investment.mapper.UserTableMapper;
import com.sztzjy.fund_investment.service.tea.TeaTopicManageService;
import com.sztzjy.fund_investment.util.PageUtil;
import com.sztzjy.fund_investment.util.ResultEntity;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Author xcj
 * @Date 2023/12/1
 */
@Service
public class TeaTopicManageServiceImpl implements TeaTopicManageService {
    @Autowired
    private TopicsMapper topicsMapper;
    @Autowired
    private QuestionAnswerMapper questionAnswerMapper;
    @Autowired
    private UserTableMapper userTableMapper;
    @Autowired
    private FlowMapper flowMapper;


    @Override
    public ResultEntity<String> insertTopics(TopicsWithBLOBs topics) {
        if (StringUtils.isBlank(topics.getTopicContent())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题干不能为空！");
        }
        if (StringUtils.isBlank(topics.getChoicesa()) || StringUtils.isBlank(topics.getChoicesb().trim())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "选项不能为空！");
        }
        if (StringUtils.isBlank(topics.getTopicType())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "题型不能为空！");
        }
        if (StringUtils.isBlank(topics.getAnswer())) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "答案不能为空！");
        }
        if (topics.getTopicType().equals("判断题")) {
            if (!topics.getAnswer().equals("A") && !topics.getAnswer().equals("B")) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "判断题答案必须为A或者B！");
            }
        }
        if (topics.getTopicType().equals("单选题")) {
            if (!topics.getAnswer().matches("[ABCDE]")) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "单选题答案必须为ABCDE中的一个！");
            }
        }
        if (topics.getTopicType().equals("多选题")) {
            if (!topics.getAnswer().matches("^[A-E](,[A-E]){1,}$")) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "多选题答案格式不正确！答案应为大写字母A到E，用英文逗号隔开，并至少选择两个选项。");
            }
            List<String> list = Arrays.asList(topics.getChoicesa(), topics.getChoicesb(), topics.getChoicesc());
            if (list.stream().anyMatch(StringUtils::isBlank)) {
                return new ResultEntity<>(HttpStatus.BAD_REQUEST, "多选题最少需要A,B,C三个选项！");
            }
        }
        if (topics.getScore() == null || topics.getScore() == 0) {
            return new ResultEntity<>(HttpStatus.BAD_REQUEST, "分值不能为空！");
        }
        topicsMapper.insert(topics);
        return new ResultEntity<>(HttpStatus.OK, "新增成功！");
    }


    /* 试题页面模糊查询/展示
     * @author xcj
     * @Date 2023/12/1
     */
    @Override
    public PageInfo<TopicsWithBLOBs> getTopicsBySchoolId(Integer index, Integer size, String schoolId, String module, String topicContent) {
        List<TopicsWithBLOBs> topics = topicsMapper.getTopicsByCondition(schoolId, module, topicContent);
        PageInfo pageInfo = PageUtil.pageHelper(topics, index, size);
        return pageInfo;
    }


    /**
     * 互动答疑查询展示
     *
     * @author xcj
     * @Date 2023/12/1
     */
    @Override
    public PageInfo<QuestionAnswerDto> getQuestionAnswer(Integer index, Integer size, String schoolId, String className, String keyWord) {
        QuestionAnswerExample questionAnswerExample = new QuestionAnswerExample();
        questionAnswerExample.createCriteria().andSchoolidEqualTo(schoolId);
        List<QuestionAnswer> questionAnswers = questionAnswerMapper.selectByExample(questionAnswerExample);
        UserTableExample userTableExample = new UserTableExample();
        UserTableExample.Criteria criteria = userTableExample.createCriteria();
        UserTableExample.Criteria or = userTableExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        if (StringUtils.isNotBlank(className)) {
            criteria.andClassNameEqualTo(className);
        }
        if (StringUtils.isNotBlank(keyWord)) {
            criteria.andStudentIdEqualTo(keyWord);
            or.andNameEqualTo(keyWord);
        }
        userTableExample.or(or);
        List<UserTable> userTables = userTableMapper.selectByExample(userTableExample);
        List<QuestionAnswerDto> list = new ArrayList<>();

        for (UserTable userTable : userTables) {
            Flow flow = flowMapper.selectByPrimaryKey(userTable.getUserid());
            String flowId = flow.getFlowId();
            for (QuestionAnswer questionAnswer : questionAnswers) {
                if (questionAnswer.getFlowId().equals(flowId)) {
                    QuestionAnswerDto questionAnswerDto = new QuestionAnswerDto();
                    BeanUtils.copyProperties(userTable, questionAnswerDto);
                    BeanUtils.copyProperties(questionAnswer, questionAnswerDto);
                    list.add(questionAnswerDto);
                }
            }
        }
        PageInfo pageInfo = PageUtil.pageHelper(list, index, size);
        return pageInfo;
    }
}
