package com.sztzjy.fund_investment.task;

import cn.hutool.core.util.IdUtil;
import com.sztzjy.fund_investment.entity.*;
import com.sztzjy.fund_investment.mapper.PerformanceScoreMapper;
import com.sztzjy.fund_investment.mapper.TeaClassScoreMapper;
import com.sztzjy.fund_investment.mapper.UserTableMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;

/**
 * @Author xcj
 * @Date 2023/11/23
 */
@Component
public class Task {
    @Autowired
    private PerformanceScoreMapper performanceScoreMapper;
    @Autowired
    private TeaClassScoreMapper teaClassScoreMapper;
    @Autowired
    private UserTableMapper userMapper;

    /* 计算每天每个班级的平均成绩 每天00.00执行
     * @author xcj
     * @Date 2023/11/23
     */
    //todo 重新实训需要把performance_score表的数据删掉，计算班级平均分时只取最新的数据
    @Scheduled(cron = "0 0 0 * * ?")
    public void getClassScore() {
        // 查到单个班级下所有的已经在投资报告页面提交过的学生的总成绩
        // 算平均值，存表
        //查出所有数据
        PerformanceScoreExample performanceScoreExample = new PerformanceScoreExample();
        List<PerformanceScore> performanceScores = performanceScoreMapper.selectByExample(performanceScoreExample);
        List<String> classIdList = new ArrayList<>();
        for (PerformanceScore performanceScore : performanceScores) {
            //查询classID相同的所有数据
            String classId = performanceScore.getClassId();
            classIdList.add(classId);
        }
        for (String classId : classIdList) {
            BigDecimal avgScore = BigDecimal.ZERO;
            BigDecimal count = BigDecimal.ZERO;
            PerformanceScoreExample performanceScoreExample2 = new PerformanceScoreExample();
            performanceScoreExample2.createCriteria().andClassIdEqualTo(classId);
            List<PerformanceScore> performanceScores2 = performanceScoreMapper.selectByExample(performanceScoreExample2);
            // 创建存储等级人数的Map
            Map<String, Integer> gradeCounts = new HashMap<>();
            gradeCounts.put("excellent", 0);
            gradeCounts.put("good", 0);
            gradeCounts.put("general", 0);
            gradeCounts.put("fail", 0);
            BigDecimal classMaxScore = BigDecimal.ZERO; // 最高分，默认为0
            BigDecimal classMinScore = null; // 最低分，默认为null
            String schoolId ="";
            for (PerformanceScore score : performanceScores2) {
                //第一个比较对象
                BigDecimal totalScore = score.getTotalScore();
                schoolId = score.getSchoolId();
                // 计算最高分
                if (totalScore.compareTo(classMaxScore) >= 0) {
                    classMaxScore = totalScore;
                }
                // 计算最低分
                if (classMinScore == null || totalScore.compareTo(classMinScore) <= 0) {
                    classMinScore = totalScore;
                }

                //没提交报告的 算0分跳过
                if (score.getInvestmentReportScore() == null || score.getInvestmentReportScore().equals(BigDecimal.ZERO)) {
                    continue;
                }
                count = count.add(BigDecimal.ONE);
                avgScore = avgScore.add(totalScore);

                // 计算等级人数
                if (totalScore.compareTo(BigDecimal.valueOf(90)) >= 0 && totalScore.compareTo(BigDecimal.valueOf(100)) <= 0) {
                    gradeCounts.put("excellent", gradeCounts.get("excellent") + 1);
                } else if (totalScore.compareTo(BigDecimal.valueOf(80)) >= 0 && totalScore.compareTo(BigDecimal.valueOf(89)) <= 0) {
                    gradeCounts.put("good", gradeCounts.get("good") + 1);
                } else if (totalScore.compareTo(BigDecimal.valueOf(60)) >= 0 && totalScore.compareTo(BigDecimal.valueOf(79)) <= 0) {
                    gradeCounts.put("general", gradeCounts.get("general") + 1);
                } else if (totalScore.compareTo(BigDecimal.valueOf(0)) >= 0 && totalScore.compareTo(BigDecimal.valueOf(59)) <= 0) {
                    gradeCounts.put("fail", gradeCounts.get("fail") + 1);
                }
            }
            TeaClassScore teaClassScore = new TeaClassScore();
            //时间为年月日
            LocalDate currentDate = LocalDate.now();
            Date date = Date.from(currentDate.atStartOfDay(ZoneId.systemDefault()).toInstant());
            teaClassScore.setStartTime(date);
            teaClassScore.setId(IdUtil.simpleUUID());
            teaClassScore.setClassMaxScore(classMaxScore);
            teaClassScore.setClassMinScore(classMinScore);
            teaClassScore.setExcellentCount(gradeCounts.get("excellent"));
            teaClassScore.setGoodCount(gradeCounts.get("good"));
            teaClassScore.setGeneralCount(gradeCounts.get("general"));
            teaClassScore.setFailCount(gradeCounts.get("fail"));
            teaClassScore.setClassId(classId);
            UserTableExample userTableExample =new UserTableExample();
            userTableExample.createCriteria().andSchoolIdEqualTo(schoolId).andClassIdEqualTo(classId);
            List<UserTable> userTables = userMapper.selectByExample(userTableExample);
            UserTable userTable = userTables.get(0);
            teaClassScore.setClassName(userTable.getClassName());
            BigDecimal divideScore = avgScore.divide(count, 2, RoundingMode.HALF_UP);
            teaClassScore.setClassAverageScore(divideScore);
            teaClassScoreMapper.insert(teaClassScore);
        }
    }
}

